using System;
using System.Collections.Generic;
using System.Text;


namespace neorbe
{
	public abstract class PatternItem
	{
		public const int ItemCode = 0;
		public const int SequenceCode = 1;
		public const int SkipCode = 2;
		public const int OpenCode = 3;
		public const int CloseCode = 4;

		public abstract int Code { get; }

		public class Item : PatternItem
		{
			public readonly byte Value;

			public Item(byte value)
			{
				this.Value = value;
			}

			public override int Code
			{
				get
				{
					return ItemCode;
				}
			}
		}

		public class Sequence : PatternItem
		{
			public readonly byte[] Value;

			public Sequence(byte[] value)
			{
				this.Value = value;
			}

			public override int Code
			{
				get
				{
					return SequenceCode;
				}
			}

			public override string ToString()
			{
				string s = string.Empty;
				foreach (byte b in Value)
					s += ((char)b).ToString();
				return "?" + s;
			}
		}

		public class Skip : PatternItem
		{
			public readonly int Size;

			public Skip(int size)
			{
				this.Size = size;
			}

			public override int Code
			{
				get
				{
					return Skip.SkipCode;
				}
			}
		}

		public class Open : PatternItem
		{
			public override int Code
			{
				get
				{
					return OpenCode;
				}
			}
		}

		public class Close : PatternItem
		{
			public override int Code
			{
				get
				{
					return CloseCode;
				}
			}
		}
	}
}
